@@ -46,6 +46,7 @@
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
 #include "htmlselection.h"
 #include "htmlfontmanager.h"
@@ -66,6 +67,7 @@
 #include "body.h"
 #include "spell.h"
 #include "resolver-progressive-impl.h"
+#include "html-stream-mem.h"
 
 #include "gtkhtmldebug.h"
 
@@ -80,6 +82,11 @@ static BonoboGenericFactory *factory;
 static gint active_controls = 0;
 
 #endif
+static void send_event_stream (GNOME_GtkHTML_Editor_Engine engine, 
+			       GNOME_GtkHTML_Editor_Listener listener,
+			       const char *name,
+			       const char *url,
+			       GtkHTMLStream *stream); 
 
 
 /* This is the initialization that can only be performed after the
@@ -91,7 +98,14 @@ struct _SetFrameData {
 };
 typedef struct _SetFrameData SetFrameData;
 
-GtkHTMLEditorAPI *editor_api;
+static GtkHTMLEditorAPI *editor_api;
+
+static gint
+gtk_toolbar_focus (GtkContainer     *container,
+			  GtkDirectionType  direction)
+{
+	return FALSE;
+}
 
 static void
 set_frame_cb (BonoboControl *control,
@@ -115,6 +129,10 @@ set_frame_cb (BonoboControl *control,
 	/* Setup the tool bar.  */
 
 	toolbar = toolbar_style (control_data);
+
+	/* hack to fix gtk toolbar focus bug, should be removed once gtk+ is fixed */
+	if (GTK_CONTAINER_CLASS (gtk_type_class (gtk_toolbar_get_type ()))->focus == NULL)
+		GTK_CONTAINER_CLASS (gtk_type_class (gtk_toolbar_get_type ()))->focus = gtk_toolbar_focus;
 	gtk_box_pack_start (GTK_BOX (control_data->vbox), toolbar, FALSE, FALSE, 0);
 
 	scroll_frame = e_scroll_frame_new (NULL, NULL);
@@ -128,6 +146,12 @@ set_frame_cb (BonoboControl *control,
 	/* Setup the menu bar.  */
 
 	menubar_setup (ui_component, control_data);
+
+	if (!spell_has_control ()) {
+		control_data->has_spell_control = FALSE;
+		bonobo_ui_component_set_prop (ui_component, "/commands/EditSpellCheck", "sensitive", "0", NULL);
+	} else
+		control_data->has_spell_control = TRUE;
 }
 
 static gint
@@ -150,7 +174,6 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 		}
 		if (run_dialog) {
 			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE, _("Properties"));
-
 			html_cursor_jump_to (e->cursor, e, cd->obj, 0);
 			html_engine_disable_selection (e);
 			html_engine_set_mark (e);
@@ -224,7 +247,6 @@ load_from_file (GtkHTML *html,
         const char *path;
 
         if (strncmp (url, "file:", 5) != 0) {
-		g_warning ("Unsupported image url: %s", url);
 		return FALSE;
 	} 
 	path = url + 5; 
@@ -242,7 +264,7 @@ load_from_file (GtkHTML *html,
 		/* check to see if we stopped because of an error */
 		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
 		g_warning ("%s", g_strerror (errno));
-		return FALSE;
+		return TRUE;
 	}	
 	/* done with no errors */
 	gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
@@ -250,6 +272,7 @@ load_from_file (GtkHTML *html,
 	return TRUE;
 }
 
+#if 0
 static int
 load_from_corba (BonoboControl *control,
 		 GtkHTML *html,
@@ -290,38 +313,48 @@ load_from_corba (BonoboControl *control,
 
 	return ret_val;
 }
+#endif
 
 static void
 url_requested_cb (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
-	BonoboControl *control;
+	GtkHTMLControlData *cd = (GtkHTMLControlData *)data;
 
 	g_return_if_fail (data != NULL);
 	g_return_if_fail (url != NULL);
 	g_return_if_fail (handle != NULL);
 
-	control = BONOBO_CONTROL (data);
-
-	if (load_from_corba (control, html, url, handle))
-		;
-		/* g_warning ("valid corba reponse"); */
-	else if (load_from_file (html, url, handle))
-		;
+	if (load_from_file (html, url, handle)) {
 		/* g_warning ("valid local reponse"); */
-	else
-		g_warning ("unable to resolve url: %s", url);
+		
+	} else if (cd->editor_bonobo_engine) {
+		GNOME_GtkHTML_Editor_Engine engine;
+		GNOME_GtkHTML_Editor_Listener listener;
+		CORBA_Environment ev;
+		
+		CORBA_exception_init (&ev);
+		engine = bonobo_object_corba_objref (BONOBO_OBJECT (cd->editor_bonobo_engine));
 
+		if (engine != CORBA_OBJECT_NIL
+		    && (listener = GNOME_GtkHTML_Editor_Engine__get_listener (engine, &ev)) != CORBA_OBJECT_NIL) {
+			send_event_stream (engine, listener, "url_requested", url, handle);
+		}	
+		CORBA_exception_free (&ev);
+	} else {
+		g_warning ("unable to resolve url: %s", url);
+	}
 }
 
 static gint
 html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData *cd)
 {
 	HTMLEngine *engine = cd->html->engine;
+	guint offset;
 
 	cd->obj = html_engine_get_object_at (engine,
 					     event->x + engine->x_offset,
 					     event->y + engine->y_offset,
-					     NULL, FALSE);
+					     &offset, FALSE);
 	switch (event->button) {
 	case 1:
 		if (event->type == GDK_2BUTTON_PRESS && cd->obj && event->state & GDK_CONTROL_MASK)
@@ -334,6 +367,14 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 		/* pass this for pasting */
 		return TRUE;
 	case 3:
+		if (!html_engine_is_selection_active (engine) || !html_engine_point_in_selection (engine, cd->obj, offset)) {
+			html_engine_disable_selection (engine);
+			html_engine_jump_at (engine,
+					     event->x + engine->x_offset,
+					     event->y + engine->y_offset);
+			gtk_html_update_styles (cd->html);
+		}
+
 		if (popup_show (cd, event))
 			gtk_signal_emit_stop_by_name (GTK_OBJECT (html), "button_press_event");
 		break;
@@ -375,7 +416,8 @@ editor_init_painters (GtkHTMLControlData *cd)
 		cd->plain_painter = HTML_GDK_PAINTER (html_plain_painter_new (TRUE));
 		html_font_manager_set_default (&HTML_PAINTER (cd->plain_painter)->font_manager,
 					       prop->font_var,      prop->font_fix,
-					       prop->font_var_size, prop->font_fix_size);
+					       prop->font_var_size, prop->font_var_points,
+					       prop->font_fix_size, prop->font_fix_points);
 		
 		html_colorset_add_slave (html->engine->settings->color_set, 
 					 HTML_PAINTER (cd->plain_painter)->color_set);
@@ -454,7 +496,7 @@ editor_set_prop (BonoboPropertyBag *bag,
 {
 	GtkHTMLControlData *cd = user_data;
 	
-	g_warning ("set_prop");
+	/* g_warning ("set_prop"); */
 	switch (arg_id) {
 	case PROP_EDIT_HTML:
 		editor_set_format (cd, BONOBO_ARG_GET_BOOLEAN (arg));
@@ -533,7 +575,7 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 			    GTK_SIGNAL_FUNC (destroy_control_data_cb), cd);
 
 	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
-			    GTK_SIGNAL_FUNC (url_requested_cb), control);
+			    GTK_SIGNAL_FUNC (url_requested_cb), cd);
 
 	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
 			    GTK_SIGNAL_FUNC (html_button_pressed), cd);
@@ -586,6 +628,76 @@ wait_for_bonobo_patch__bonobo_arg_type_to_gtk (BonoboArgType id)
 	return gtk_type;
 }
 
+static GtkArg *
+send_event_str (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listener listener, gchar *name, GtkArg *arg)
+{
+	CORBA_Environment ev;
+	GtkArg *gtk_retval = NULL;
+	BonoboArg *bonobo_arg = bonobo_arg_new (bonobo_arg_type_from_gtk (arg->type));
+	BonoboArg *bonobo_retval;
+
+	bonobo_arg_from_gtk (bonobo_arg, arg);
+
+	/* printf ("sending to listener\n"); */
+	CORBA_exception_init (&ev);
+	bonobo_retval = GNOME_GtkHTML_Editor_Listener_event (listener, name, bonobo_arg, &ev);
+	CORBA_free (bonobo_arg);
+
+	if (ev._major == CORBA_NO_EXCEPTION) {
+		if (!bonobo_arg_type_is_equal (bonobo_retval->_type, TC_null, &ev)
+		    && !bonobo_arg_type_is_equal (bonobo_retval->_type, TC_void, &ev)) {
+			gtk_retval = gtk_arg_new (wait_for_bonobo_patch__bonobo_arg_type_to_gtk (bonobo_retval->_type));
+			bonobo_arg_to_gtk (gtk_retval, bonobo_retval);
+		}
+		CORBA_free (bonobo_retval);
+	}
+	CORBA_exception_free (&ev);
+
+	return gtk_retval;
+}
+
+static void
+send_event_void (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listener listener, gchar *name)
+{
+	CORBA_Environment ev;
+	CORBA_any *any;
+
+	any = CORBA_any_alloc ();
+	any->_type = TC_null;
+	CORBA_exception_init (&ev);
+	GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+	CORBA_exception_free (&ev);
+	CORBA_free (any);
+}
+
+static void
+send_event_stream (GNOME_GtkHTML_Editor_Engine engine, 
+		   GNOME_GtkHTML_Editor_Listener listener,
+		   const char *name,
+		   const char *url,
+		   GtkHTMLStream *stream) 
+{
+	CORBA_any *any;
+	GNOME_GtkHTML_Editor_URLRequestEvent e;
+	CORBA_Environment ev;
+	BonoboStream *bstream;
+
+	any = CORBA_any__alloc ();
+	any->_type = TC_GNOME_GtkHTML_Editor_URLRequestEvent;
+	any->_value = &e;
+
+	e.url = (char *)url;
+	bstream = html_stream_mem_create (stream);
+	e.stream = BONOBO_OBJREF (bstream);
+	
+	CORBA_exception_init (&ev);
+	GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+
+	bonobo_object_unref (BONOBO_OBJECT (bstream));
+	CORBA_exception_free (&ev);
+	CORBA_free (any);
+}
+		  
 static GtkArg *
 editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data)
 {
@@ -594,39 +706,32 @@ editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **arg
 
 	/* printf ("editor_api_event\n"); */
 
-	if (event_type != GTK_HTML_EDITOR_EVENT_COMMAND && event_type != GTK_HTML_EDITOR_EVENT_IMAGE_URL) {
-		g_warning ("Now, only COMMAND and IMAGE_URL events are supported.\n");
-		return NULL;
-	}
 	if (cd->editor_bonobo_engine) {
 		GNOME_GtkHTML_Editor_Engine engine;
 		GNOME_GtkHTML_Editor_Listener listener;
 		CORBA_Environment ev;
 
+		CORBA_exception_init (&ev);
 		engine = bonobo_object_corba_objref (BONOBO_OBJECT (cd->editor_bonobo_engine));
+
 		if (engine != CORBA_OBJECT_NIL
 		    && (listener = GNOME_GtkHTML_Editor_Engine__get_listener (engine, &ev)) != CORBA_OBJECT_NIL) {
 
-			BonoboArg *arg = bonobo_arg_new (bonobo_arg_type_from_gtk (args [0]->type));
-			BonoboArg *bonobo_retval;
-
-			bonobo_arg_from_gtk (arg, args [0]);
-
-			/* printf ("sending to listener\n"); */
-			CORBA_exception_init (&ev);
-			bonobo_retval = GNOME_GtkHTML_Editor_Listener_event (listener,
-									 event_type ==  GTK_HTML_EDITOR_EVENT_COMMAND
-									 ? "command" : "image_url",
-									 arg, &ev);
-			CORBA_free (arg);
-
-			if (ev._major == CORBA_NO_EXCEPTION) {
-				if (!bonobo_arg_type_is_equal (bonobo_retval->_type, TC_null, &ev)
-				    && !bonobo_arg_type_is_equal (bonobo_retval->_type, TC_void, &ev)) {
-					gtk_retval = gtk_arg_new (wait_for_bonobo_patch__bonobo_arg_type_to_gtk (bonobo_retval->_type));
-					bonobo_arg_to_gtk (gtk_retval, bonobo_retval);
-				}
-				CORBA_free (bonobo_retval);
+			switch (event_type) {
+			case GTK_HTML_EDITOR_EVENT_COMMAND_BEFORE:
+				gtk_retval = send_event_str (engine, listener, "command_before", args [0]);
+				break;
+			case GTK_HTML_EDITOR_EVENT_COMMAND_AFTER:
+				gtk_retval = send_event_str (engine, listener, "command_after", args [0]);
+				break;
+			case GTK_HTML_EDITOR_EVENT_IMAGE_URL:
+				gtk_retval = send_event_str (engine, listener, "image_url", args [0]);
+				break;
+			case GTK_HTML_EDITOR_EVENT_DELETE:
+				send_event_void (engine, listener, "delete");
+				break;
+			default:
+				g_warning ("Unsupported event.\n");
 			}
 			CORBA_exception_free (&ev);
 		}
@@ -656,6 +761,7 @@ new_editor_api ()
 	editor_api->suggestion_request = spell_suggestion_request;
 	editor_api->add_to_personal    = spell_add_to_personal;
 	editor_api->add_to_session     = spell_add_to_session;
+	editor_api->set_language       = spell_set_language;
 	editor_api->command            = editor_api_command;
 	editor_api->event              = editor_api_event;
 	editor_api->create_input_line  = editor_api_create_input_line;
@@ -664,12 +770,16 @@ new_editor_api ()
 static void
 editor_control_init (void)
 {
-	new_editor_api ();
-	gdk_rgb_init ();
-	glade_gnome_init ();
-}
+	static gboolean initialized = FALSE;
 
-static gboolean editor_control_initialized = FALSE;
+	if (!initialized) {
+		initialized = TRUE;
+
+		new_editor_api ();
+		gdk_rgb_init ();
+		glade_gnome_init ();
+	}
+}
 
 static BonoboObject *
 editor_control_factory (BonoboGenericFactory *factory, gpointer closure)
@@ -677,8 +787,7 @@ editor_control_factory (BonoboGenericFactory *factory, gpointer closure)
 	BonoboControl *control;
 	GtkWidget *vbox;
 
-	if (!editor_control_initialized)
-		editor_control_init ();
+	editor_control_init ();
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
@@ -708,7 +817,7 @@ BONOBO_OAF_SHLIB_FACTORY ("OAFIID:GNOME_GtkHTML_Editor_Factory", "GNOME HTML Edi
 static CORBA_ORB
 init_corba (int *argc, char **argv)
 {
-	gnome_init_with_popt_table ("html-editor-factory", "0.0",
+	gnome_init_with_popt_table ("gnome-gtkhtml-editor", VERSION,
 				    *argc, argv, oaf_popt_options, 0, NULL);
 
 	return oaf_init (*argc, argv);
