@@ -68,22 +68,19 @@ ps_impl_load (BonoboPersistStream *ps,
 	do {
 		Bonobo_Stream_read (stream, READ_CHUNK_SIZE,
 				    &buffer, ev);
-		if (ev->_major != CORBA_NO_EXCEPTION)
+		if ((ev->_major != CORBA_NO_EXCEPTION)
+		    || (buffer->_length <= 0)) {
+		        CORBA_free (buffer);
 			break;
+		}
 
-		if (buffer->_length <= 0)
-			break;
 		gtk_html_write (html, handle, buffer->_buffer, buffer->_length);
 		CORBA_free (buffer);
 	} while (1);
 
-	if (ev->_major != CORBA_NO_EXCEPTION) {
-		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
-		bonobo_persist_stream_set_dirty (ps, TRUE);
-	} else {
-		CORBA_free (buffer);
-		gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
-	}
+	bonobo_persist_stream_set_dirty (ps, TRUE);
+	gtk_html_end (html, handle, 
+		      (ev->_major == CORBA_NO_EXCEPTION) ? GTK_HTML_STREAM_OK : GTK_HTML_STREAM_ERROR);
 
 	if (was_editable)
 		gtk_html_set_editable (html, TRUE);
@@ -108,6 +105,10 @@ save_receiver (const HTMLEngine *engine,
 	SaveState *state;
 
 	state = (SaveState *) user_data;
+	if (state->stream == CORBA_OBJECT_NIL)
+		CORBA_exception_set (state->ev, CORBA_USER_EXCEPTION,
+				     ex_Bonobo_IOError, NULL);
+
 	if (state->ev->_major != CORBA_NO_EXCEPTION)
 		return FALSE;
 
@@ -193,3 +194,6 @@ persist_stream_impl_new (GtkHTML *html)
 
 
 
+
+
+
