@@ -23,9 +23,14 @@
 #include <config.h>
 #include <gal/widgets/e-scroll-frame.h>
 
+#include "gtkhtml.h"
+#include "gtkhtml-properties.h"
+
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
+#include "htmlengine-edit.h"
+#include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
 #include "htmlobject.h"
@@ -64,7 +69,7 @@ destroy (GtkWidget *w, SpellPopup *sp)
 
 			gtk_clist_get_text (GTK_CLIST (sp->clist),
 					    GPOINTER_TO_INT (GTK_CLIST (sp->clist)->selection->data), 0, &replacement);
-			html_engine_replace_word_with (sp->cd->html->engine, replacement);
+			html_engine_replace_spell_word_with (sp->cd->html->engine, replacement);
 
 			/* printf ("replace: %s with: %s\n", sp->misspeled_word, replacement); */
 		}
@@ -201,6 +206,7 @@ spell_check_word (GtkHTML *html, const gchar *word, gpointer data)
 	if (!cd->dict)
 		return TRUE;
 
+	/* printf ("check word: %s\n", word); */
 	CORBA_exception_init (&ev);
 	rv = GNOME_Spell_Dictionary_checkWord (cd->dict, word, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION)
@@ -216,6 +222,8 @@ spell_add_to_session (GtkHTML *html, const gchar *word, gpointer data)
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
 	CORBA_Environment   ev;
 
+	g_return_if_fail (word);
+
 	if (!cd->dict)
 		return;
 
@@ -230,6 +238,8 @@ spell_add_to_personal (GtkHTML *html, const gchar *word, gpointer data)
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
 	CORBA_Environment   ev;
 
+	g_return_if_fail (word);
+
 	if (!cd->dict)
 		return;
 
@@ -238,15 +248,43 @@ spell_add_to_personal (GtkHTML *html, const gchar *word, gpointer data)
 	CORBA_exception_free (&ev);
 }
 
+void
+spell_set_language (GtkHTML *html, const gchar *language, gpointer data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
+	CORBA_Environment   ev;
+
+	if (!cd->dict)
+		return;
+
+	CORBA_exception_init (&ev);
+	GNOME_Spell_Dictionary_setTag (cd->dict, "language-tag", language, &ev);
+	CORBA_exception_free (&ev);
+}
+
+void
+spell_init (GtkHTML *html, GtkHTMLControlData *cd)
+{
+	CORBA_Environment   ev;
+
+	if (!cd->dict)
+		return;
+
+	CORBA_exception_init (&ev);
+	/* GNOME_Spell_Dictionary_setTag (cd->dict, "sug-mode", "fast", &ev); */
+	GNOME_Spell_Dictionary_setTag (cd->dict, "encoding", "utf-8", &ev);
+	CORBA_exception_free (&ev);
+}
+
 static void
 set_word (GtkHTMLControlData *cd)
 {
 	CORBA_Environment ev;
 
 	CORBA_exception_init (&ev);
-	gtk_html_select_word (cd->html);
+	html_engine_select_spell_word_editable (cd->html->engine);
 	bonobo_property_bag_client_set_value_string (cd->spell_control_pb, "word",
-						     html_engine_get_word (cd->html->engine), &ev);
+						     html_engine_get_spell_word (cd->html->engine), &ev);
 	CORBA_exception_free (&ev);
 }
 
@@ -254,7 +292,8 @@ static gboolean
 next_word (GtkHTMLControlData *cd)
 {
 	gboolean rv = TRUE;
-	while (html_engine_forward_word (cd->html->engine) && (rv = html_engine_word_is_valid (cd->html->engine)))
+	while (html_engine_forward_word (cd->html->engine)
+	       && (rv = html_engine_spell_word_is_valid (cd->html->engine)))
 		;
 
 	return rv;
@@ -265,12 +304,12 @@ check_next_word (GtkHTMLControlData *cd, gboolean update)
 {
 	HTMLEngine *e = cd->html->engine;
 
-	if (update && e->cursor->object->parent && HTML_IS_CLUEFLOW (e->cursor->object->parent))
-		html_clueflow_spell_check (HTML_CLUEFLOW (e->cursor->object->parent), e, NULL);
+	html_engine_disable_selection (e);
+	if (update)
+		html_engine_spell_check (e);
 
-	if (next_word (cd)) {
+	if (!cd->spell_check_next || next_word (cd)) {
 		gnome_dialog_close (GNOME_DIALOG (cd->spell_dialog));
-		html_engine_disable_selection (e);
 	} else {
 		set_word (cd);
 	}
@@ -285,12 +324,14 @@ replace_cb (BonoboListener    *listener,
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
 
-	html_engine_replace_word_with (cd->html->engine, BONOBO_ARG_GET_STRING (arg));
+	/* printf ("replace '%s'\n", BONOBO_ARG_GET_STRING (arg)); */
+
+	html_engine_replace_spell_word_with (cd->html->engine, BONOBO_ARG_GET_STRING (arg));
 	check_next_word (cd, FALSE);
 }
 
 static void 
-next_cb (BonoboListener    *listener,
+skip_cb (BonoboListener    *listener,
 	    char              *event_name, 
 	    CORBA_any         *arg,
 	    CORBA_Environment *ev,
@@ -300,12 +341,38 @@ next_cb (BonoboListener    *listener,
 }
 
 static void 
-next_update_cb (BonoboListener    *listener,
-		char              *event_name, 
-		CORBA_any         *arg,
-		CORBA_Environment *ev,
-		gpointer           user_data)
+add_cb (BonoboListener    *listener,
+	char              *event_name, 
+	CORBA_any         *arg,
+	CORBA_Environment *ev,
+	gpointer           user_data)
 {
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
+	gchar *word;
+
+	word = html_engine_get_spell_word (cd->html->engine);
+	g_return_if_fail (word);
+
+	GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, word, ev);
+	g_free (word);
+	check_next_word ((GtkHTMLControlData *) user_data, TRUE);
+}
+
+static void 
+ignore_cb (BonoboListener    *listener,
+	   char              *event_name, 
+	   CORBA_any         *arg,
+	   CORBA_Environment *ev,
+	   gpointer           user_data)
+{
+	GtkHTMLControlData *cd = (GtkHTMLControlData *) user_data;
+	gchar *word;
+
+	word = html_engine_get_spell_word (cd->html->engine);
+	g_return_if_fail (word);
+
+	GNOME_Spell_Dictionary_addWordToSession (cd->dict, word, ev);
+	g_free (word);
 	check_next_word ((GtkHTMLControlData *) user_data, TRUE);
 }
 
@@ -325,14 +392,29 @@ spell_has_control ()
 }
 
 void
-spell_check_document (GtkHTMLControlData *cd)
+spell_check_dialog (GtkHTMLControlData *cd, gboolean whole_document)
 {
 	GtkWidget *control;
 	GtkWidget *dialog;
+	guint position;
+
+	position = cd->html->engine->cursor->position;
+	cd->spell_check_next = whole_document;
+	if (whole_document) {
+		html_engine_disable_selection (cd->html->engine);
+		html_engine_beginning_of_document (cd->html->engine);
+	}
+
+	if (html_engine_spell_word_is_valid (cd->html->engine))
+		if (next_word (cd)) {
+			html_engine_hide_cursor (cd->html->engine);
+			html_cursor_jump_to_position (cd->html->engine->cursor, cd->html->engine, position);
+			html_engine_show_cursor (cd->html->engine);
+
+			gnome_ok_dialog (_("No misspelled word found"));
 
-	if (html_engine_word_is_valid (cd->html->engine))
-		if (next_word (cd))
 			return;
+		}
 
 	dialog  = gnome_dialog_new (_("Spell checker"), GNOME_STOCK_BUTTON_CLOSE, NULL);
 	control = bonobo_widget_new_control (CONTROL_IID, CORBA_OBJECT_NIL);
@@ -346,11 +428,14 @@ spell_check_document (GtkHTMLControlData *cd)
 	cd->spell_dialog = dialog;
         cd->spell_control_pb = bonobo_control_frame_get_control_property_bag
 		(bonobo_widget_get_control_frame (BONOBO_WIDGET (control)), NULL);
+	bonobo_property_bag_client_set_value_string (cd->spell_control_pb, "language",
+						     GTK_HTML_CLASS (GTK_OBJECT (cd->html)->klass)->properties->language,
+						     NULL);
 	bonobo_event_source_client_add_listener (cd->spell_control_pb, replace_cb, "Bonobo/Property:change:replace", NULL, cd);
-	bonobo_event_source_client_add_listener (cd->spell_control_pb, next_update_cb, "Bonobo/Property:change:add", NULL, cd);
-	bonobo_event_source_client_add_listener (cd->spell_control_pb, next_update_cb, "Bonobo/Property:change:ignore",
+	bonobo_event_source_client_add_listener (cd->spell_control_pb, add_cb, "Bonobo/Property:change:add", NULL, cd);
+	bonobo_event_source_client_add_listener (cd->spell_control_pb, ignore_cb, "Bonobo/Property:change:ignore",
 						 NULL, cd);
-	bonobo_event_source_client_add_listener (cd->spell_control_pb, next_cb, "Bonobo/Property:change:skip", NULL, cd);
+	bonobo_event_source_client_add_listener (cd->spell_control_pb, skip_cb, "Bonobo/Property:change:skip", NULL, cd);
 	set_word (cd);
 
 	gtk_widget_show (control);
