@@ -33,10 +33,15 @@
 #include "htmlsearch.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
+#include "htmltokenizer.h"
+
+#ifndef USE_SCROLLED_WINDOW
+#include <gal/widgets/e-scroll-frame.h>
+#endif
 
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
-static gboolean calc_size (HTMLObject *o, HTMLPainter *painter);
+static gboolean calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
 
 static void
 iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
@@ -185,24 +190,22 @@ draw (HTMLObject *o,
 	ArtIRect paint;
 
 	if (GTK_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
+		gint pixel_size = html_painter_get_pixel_size (e->painter);
 		html_object_calc_intersection (o, &paint, x, y, width, height);
 		if (art_irect_empty (&paint))
 			return;
 
-		if (GTK_OBJECT_TYPE (e->painter) != HTML_TYPE_GDK_PAINTER) {
-			gint pixel_size = html_painter_get_pixel_size (e->painter);
-			html_object_draw (e->clue, e->painter,
-					  x, y,
-					  width - pixel_size * (e->leftBorder + e->rightBorder),
-					  height - pixel_size * (e->topBorder + e->bottomBorder),
-					  tx + pixel_size * e->leftBorder, ty + pixel_size * e->topBorder);
-		}
+		html_object_draw (e->clue, e->painter,
+				  x, y,
+				  width - pixel_size * (e->leftBorder + e->rightBorder),
+				  height - pixel_size * (e->topBorder + e->bottomBorder),
+				  tx + pixel_size * e->leftBorder, ty + pixel_size * e->topBorder);
 	} else
 		(*HTML_OBJECT_CLASS (parent_class)->draw) (o, p, x, y, width, height, tx, ty);
 }
 
 static void
-set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+set_painter (HTMLObject *o, HTMLPainter *painter)
 {
 	HTMLIFrame *iframe;
 
@@ -212,8 +215,7 @@ set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	}
 	
 	html_engine_set_painter (GTK_HTML (iframe->html)->engine,
-				 GTK_OBJECT_TYPE (painter) != HTML_TYPE_PRINTER ? iframe->gdk_painter : painter,
-				 max_width);
+				 GTK_OBJECT_TYPE (painter) != HTML_TYPE_PRINTER ? iframe->gdk_painter : painter);
 }
 
 static void
@@ -235,9 +237,39 @@ check_page_split (HTMLObject *self, gint y)
 	return html_object_check_page_split (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue, y);
 }
 
+static void
+copy (HTMLObject *self,
+      HTMLObject *dest)
+{
+        HTMLIFrame *s = HTML_IFRAME (self);
+        HTMLIFrame *d = HTML_IFRAME (dest);
+
+	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
+
+	d->scroll = NULL;
+	d->html = NULL;
+	d->gdk_painter = NULL;
+
+	d->url = g_strdup (s->url);
+	d->width = s->width;
+	d->height = s->height;
+	d->frameborder = s->frameborder;
+}
+
+void
+html_iframe_set_scrolling (HTMLIFrame *iframe, GtkPolicyType scroll)
+{
+#if E_USE_SCROLLED_WINDOW
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (iframe->scroll),
+					scroll, scroll);
+#else
+	e_scroll_frame_set_policy (E_SCROLL_FRAME (iframe->scroll),
+				   scroll, scroll);
+#endif					
+}
+
 static gboolean
-calc_size (HTMLObject *o,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLIFrame *iframe;
 	HTMLEngine *e;
@@ -253,7 +285,7 @@ calc_size (HTMLObject *o,
 
 	if ((iframe->width < 0) && (iframe->height < 0)) {
 		e->width = o->max_width;
-		html_engine_calc_size (e);
+		html_engine_calc_size (e, changed_objs);
 
 		height = html_engine_get_doc_height (e);
 		width = html_engine_get_doc_width (e);
@@ -261,16 +293,13 @@ calc_size (HTMLObject *o,
 		gtk_widget_set_usize (iframe->scroll, width, height);
 		gtk_widget_queue_resize (iframe->scroll);
 		
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (iframe->scroll),
-						GTK_POLICY_NEVER,
-						GTK_POLICY_NEVER);
+		html_iframe_set_scrolling (iframe, GTK_POLICY_NEVER);
 
 		o->width = width;
 		o->ascent = height;
 		o->descent = 0;
 	} else
-		return (* HTML_OBJECT_CLASS (parent_class)->calc_size) 
-			(o, painter);
+		return (* HTML_OBJECT_CLASS (parent_class)->calc_size) (o, painter, changed_objs);
 
 	if (o->descent != old_descent
 	    || o->ascent != old_ascent
@@ -385,6 +414,8 @@ destroy (HTMLObject *o)
 
 	iframe_set_gdk_painter (iframe, NULL);
 
+	g_free (iframe->url);
+
 	if (iframe->html) {
 		gtk_signal_disconnect_by_data (GTK_OBJECT (iframe->html), o);
 		iframe->html = NULL;
@@ -403,6 +434,7 @@ html_iframe_init (HTMLIFrame *iframe,
 		  gboolean border)
 {
 	HTMLEmbedded *em = HTML_EMBEDDED (iframe);
+	HTMLTokenizer *new_tokenizer;
 	GtkWidget *new_widget;
 	GtkHTML   *new_html;
 	GtkHTML   *parent_html;
@@ -415,21 +447,35 @@ html_iframe_init (HTMLIFrame *iframe,
 	html_embedded_init (em, HTML_EMBEDDED_CLASS (klass),
 			    parent, NULL, NULL);
 	
+#if USE_SCROLLED_WINDOW
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
+#else
+	scrolled_window = e_scroll_frame_new (NULL, NULL);
+	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scrolled_window), 
+					border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
+
+#endif
+
+	iframe->scroll = scrolled_window;
+	html_iframe_set_scrolling (iframe, GTK_POLICY_AUTOMATIC);
+
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
 
+	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
+
+	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
+	gtk_object_unref (GTK_OBJECT (new_tokenizer));
+	new_tokenizer = NULL;
+
 	gtk_html_set_default_content_type (new_html,
 					   parent_html->priv->content_type);
 	iframe->html = new_widget;
-	gtk_html_set_iframe_parent (new_html, parent);
+	gtk_html_set_iframe_parent (new_html, parent, HTML_OBJECT (iframe));
 	gtk_container_add (GTK_CONTAINER (scrolled_window), new_widget);
 	gtk_widget_show (new_widget);
 
-	iframe->url = src;
+	iframe->url = g_strdup (src);
 	iframe->width = width;
 	iframe->height = height;
 	iframe->gdk_painter = NULL;
@@ -459,11 +505,9 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_emit_by_name (GTK_OBJECT (new_html->engine), 
 				 "url_requested", src, handle);
 
-	if ((width > 0) && (height > 0))
-		gtk_widget_set_usize (scrolled_window, width, height);
+	gtk_widget_set_usize (scrolled_window, width, height);
 
 	gtk_widget_show (scrolled_window);	
-	iframe->scroll = scrolled_window;
 
 	html_embedded_set_widget (em, scrolled_window);
 
@@ -475,6 +519,7 @@ html_iframe_init (HTMLIFrame *iframe,
 				     parent_html->engine->settings->color_set);
 	html_colorset_set_unchanged (new_html->engine->settings->color_set,
 				     parent_html->engine->settings->color_set);
+	html_painter_set_focus (new_html->engine->painter, parent_html->engine->have_focus);
 	/*
 	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
 			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
@@ -519,6 +564,7 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->set_painter             = set_painter;
 	object_class->reset                   = reset;
 	object_class->draw                    = draw;
+	object_class->copy                    = copy;
 	object_class->set_max_width           = set_max_width;
 	object_class->forall                  = forall;
 	object_class->check_page_split        = check_page_split;
